from structures.graph import Graph
from dijkstra import Dijkstra
from math import inf


class CityNetwork:
    def __init__(self, input_file, max_distance=inf):  # buduje graf połączeń nie dłuższych, niż max_distance
        self.g = Graph()
        with open(input_file, 'r') as f:
            cities = []
            num_cities = int(f.readline())
            for i in range(num_cities):
                city_name = f.readline().strip()
                cities.append(city_name)
                self.g.add_vertex(city_name)
            for i in range(num_cities):
                distances = f.readline().split()
                assert len(distances) == num_cities
                for j in range(num_cities):
                    dist = int(distances[j])
                    if dist > max_distance:
                        continue
                    self.g.add_edge(cities[i], cities[j], dist)
                    # self.g.add_edge(cities[j], cities[i], dist)

    def shortest_path(self, from_city, to_city):
        dij = Dijkstra(self.g)
        dij.dijkstra(from_city)
        return dij.traverse(to_city)

    def describe_route(self, from_city, to_city):  # ładny opis ścieżki
        path = self.shortest_path(from_city, to_city)
        output = []
        total = 0
        for i in range(len(path)-1):
            city1 = self.g.get_vertex(path[i])
            city2 = self.g.get_vertex(path[i + 1])
            dist = city1.get_weight(city2)
            total += dist
            output.append(f"{city1.id}-{city2.id}: {dist}")
        if output == []:
            return 'No possible route!'
        return ', '.join(output) + f' (total: {total})'


if __name__ == "__main__":
    print('Shortest Paris-Baghdad route (with capped leg distance):')
    for max_dist in (8, 9, 10, 11, 15, 20, 24):
        network = CityNetwork('cities.txt', max_dist)
        print(f'Max distance {max_dist}: ' +
              network.describe_route('Paris', 'Tokyo'))
